@@ -1,4 +1,4 @@
-# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2024 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -552,6 +552,9 @@ say_layer = "screens"
 # The layer the choice screen is shown on.
 choice_layer = "screens"
 
+# The layer the choice screen is shown on, when passed (nvl=True).
+nvl_choice_layer = "screens"
+
 # If true, we will not use the .report_traceback method to produced
 # prettier tracebacks.
 raw_tracebacks = ("RENPY_RAW_TRACEBACKS" in os.environ)
@@ -599,7 +602,10 @@ dispatch_gesture = None
 
 # The table mapping gestures to events used by the default function.
 gestures = {
-    "n_s_w_e_w_e" : "progress_screen",
+    "s_n_e_s_w" : "progress_screen",
+    "n_e_s_w" : "progress_screen",
+    "ne_se" : "accessibility",
+    "nw_sw" : "accessibility",
     }
 
 # Sizes of gesture components and strokes, as a fraction of screen_width.
@@ -683,6 +689,9 @@ quit_on_mobile_background = False
 # Should Ren'Py pass the raw joystick (not controller) events.?
 pass_joystick_events = False
 
+# Should Ren'Py pass the controller events?
+pass_controller_events = False
+
 # A list of screens that should be shown when the overlay is enabled.
 overlay_screens = [ ]
 
@@ -891,7 +900,7 @@ profile_time = 1.0 / 50.0
 # What event do we check to see if the profile needs to be printed?
 profile_to_event = "flip"
 
-# Should we instantly zap transient displayables, or properly hide them?
+# Should unhandled events be ignored?
 fast_unhandled_event = True
 
 # Should a fast path be used when displaying empty windows.
@@ -1079,7 +1088,8 @@ touch_keyboard = os.environ.get("RENPY_TOUCH_KEYBOARD", False)
 
 # The size of the framebuffer Ren'Py creates, which doubles as the
 # largest texture size.
-fbo_size = (4096, 4096)
+max_texture_size = (4096, 4096)
+fbo_size = max_texture_size
 
 # Names to ignore the redefinition of.
 lint_ignore_redefine = [ "gui.about" ]
@@ -1163,7 +1173,7 @@ scene_clears_layer_at_list = True
 mouse_displayable = None
 
 # The default bias for the GL level of detail.
-gl_lod_bias = -.5
+gl_lod_bias = -.6
 
 # A dictionary from a tag (or None) to a function that adjusts the attributes
 # of that tag.
@@ -1204,7 +1214,7 @@ input_caret_blink = 1.
 single_movie_channel = None
 
 # Should Ren'Py raise exceptions when finding an image?
-raise_image_exceptions = True
+raise_image_exceptions = None
 
 # Should the size transform property only accept numbers of pixels ?
 relative_transform_size = True
@@ -1395,6 +1405,68 @@ linear_fades = False
 # Classes that used to participate in rollback, but no longer do.
 ex_rollback_classes = [ ]
 
+# Should we revert to the old behavior of box_reverse?
+simple_box_reverse = False
+
+# If True, positional-only parameters are allowed in ATL transform signatures.
+atl_pos_only = False
+
+# A map from font name to the hinting for the font.
+font_hinting = { None : "auto" }
+
+# Should ATL interpolation handle mixed position types,
+# at the cost of returning instances of the position type ?
+mixed_position = True
+
+# Should text interpolations be treated as Python expressions?
+interpolate_exprs = True
+
+# Should we execute costly tasks which are
+# avoidable when not generating the documentation ?
+generating_documentation = False
+
+# Is the user allowed to save, or to see the existing save files?
+save = True
+
+# Should the persistent file be updated?
+save_persistent = True
+
+# Should new drags be added to the top of a drag group.
+drag_group_add_top = True
+
+# Should loading of tl scripts be deferred?
+defer_tl_scripts = False
+
+# Should transitions take placement from child displayables?
+transitions_use_child_placement = True
+
+# Which transform events should containers pass to their children?
+containers_pass_transform_events = {'hover', 'idle', 'insensitive', 'selected_hover', 'selected_idle' }
+
+# Should the say screens be given the replace event for the second and
+# later pauses?
+say_replace_event = True
+
+# Will screens never cancel hide and replace events?
+screens_never_cancel_hide = True
+
+# A list of transforms that are applied to entire layers.
+layer_transforms = { }
+
+# True if xfill or yfill can cause a window to shrink.
+fill_shrinks_frame = False
+
+# Set this to true to log events to log.txt.
+log_events = os.environ.get("RENPY_LOG_EVENTS", False)
+
+# Callbacks to run just before python exits.
+python_exit_callbacks = [ ]
+
+# Should exceptions be raised if an image fails to load.
+raise_image_load_exceptions = None
+
+# 8.2.2
+
 del os
 del collections
 
@@ -1428,3 +1500,15 @@ def init():
     gl_blend_func["multiply"] = (GL_FUNC_ADD, GL_DST_COLOR, GL_ONE_MINUS_SRC_ALPHA, GL_FUNC_ADD, GL_ZERO, GL_ONE)
     gl_blend_func["min"] = (GL_MIN, GL_ONE, GL_ONE, GL_MIN, GL_ONE, GL_ONE)
     gl_blend_func["max"] = (GL_MAX, GL_ONE, GL_ONE, GL_MAX, GL_ONE, GL_ONE)
+
+
+def post_init():
+    """
+    Called after all init scripts have been run.
+    """
+
+    if renpy.config.raise_image_exceptions is None:
+        renpy.config.raise_image_exceptions = renpy.config.developer
+
+    if renpy.config.raise_image_load_exceptions:
+        renpy.config.raise_image_load_exceptions = renpy.config.developer
